"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = merge;

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../../../consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _componentsList() {
  const data = _interopRequireDefault(require("../../../consumer/component/components-list"));

  _componentsList = function () {
    return data;
  };

  return data;
}

function _lanesIsDisabled() {
  const data = require("../../../consumer/lanes/exceptions/lanes-is-disabled");

  _lanesIsDisabled = function () {
    return data;
  };

  return data;
}

function _mergeLanes() {
  const data = require("../../../consumer/lanes/merge-lanes");

  _mergeLanes = function () {
    return data;
  };

  return data;
}

function _mergeVersion() {
  const data = require("../../../consumer/versions-ops/merge-version");

  _mergeVersion = function () {
    return data;
  };

  return data;
}

function _mergeSnaps() {
  const data = require("../../../consumer/versions-ops/merge-version/merge-snaps");

  _mergeSnaps = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _hasWildcard() {
  const data = _interopRequireDefault(require("../../../utils/string/has-wildcard"));

  _hasWildcard = function () {
    return data;
  };

  return data;
}

async function merge(values, mergeStrategy, abort, resolve, idIsLane, noSnap, message, existingOnWorkspaceOnly, build) {
  const consumer = await (0, _consumer().loadConsumer)();

  if (consumer.isLegacy && (idIsLane || existingOnWorkspaceOnly || noSnap || message || abort || resolve)) {
    throw new (_lanesIsDisabled().LanesIsDisabled)();
  }

  let mergeResults;

  const firstValue = _ramda().default.head(values);

  if (resolve) {
    mergeResults = await (0, _mergeSnaps().resolveMerge)(consumer, values, message, build);
  } else if (abort) {
    mergeResults = await (0, _mergeSnaps().abortMerge)(consumer, values);
  } else if (idIsLane) {
    if (!firstValue) throw new (_generalError().default)('please specify lane name');
    const remote = values.length === 2 ? firstValue : null;
    const laneName = values.length === 2 ? values[1] : firstValue;
    mergeResults = await (0, _mergeLanes().mergeLanes)({
      consumer,
      remoteName: remote,
      laneName,
      mergeStrategy,
      noSnap,
      snapMessage: message,
      existingOnWorkspaceOnly,
      build
    });
  } else if (!_bitId().BitId.isValidVersion(firstValue)) {
    const bitIds = getComponentsToMerge(consumer, values); // @todo: version could be the lane only or remote/lane

    mergeResults = await (0, _mergeSnaps().mergeComponentsFromRemote)(consumer, bitIds, mergeStrategy, noSnap, message, build);
  } else {
    const version = firstValue;

    const ids = _ramda().default.tail(values);

    const bitIds = getComponentsToMerge(consumer, ids);
    mergeResults = await (0, _mergeVersion().mergeVersion)(consumer, version, bitIds, mergeStrategy);
  }

  await consumer.onDestroy();
  return mergeResults;
}

function getComponentsToMerge(consumer, ids) {
  if ((0, _hasWildcard().default)(ids)) {
    const componentsList = new (_componentsList().default)(consumer);
    return componentsList.listComponentsByIdsWithWildcard(ids);
  }

  return ids.map(id => consumer.getParsedId(id));
}