"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = fetch;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _pMapSeries() {
  const data = _interopRequireDefault(require("p-map-series"));

  _pMapSeries = function () {
    return data;
  };

  return data;
}

function _lodash() {
  const data = require("lodash");

  _lodash = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _hooks() {
  const data = _interopRequireDefault(require("../../../hooks"));

  _hooks = function () {
    return data;
  };

  return data;
}

function _laneId() {
  const data = require("../../../lane-id/lane-id");

  _laneId = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _scope() {
  const data = require("../../../scope");

  _scope = function () {
    return data;
  };

  return data;
}

function _scopeComponentsImporter() {
  const data = _interopRequireDefault(require("../../../scope/component-ops/scope-components-importer"));

  _scopeComponentsImporter = function () {
    return data;
  };

  return data;
}

function _objects() {
  const data = require("../../../scope/objects");

  _objects = function () {
    return data;
  };

  return data;
}

function _objectList() {
  const data = require("../../../scope/objects/object-list");

  _objectList = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const HooksManagerInstance = _hooks().default.getInstance();

async function fetch(path, ids, // ids type are determined by the fetchOptions.type
fetchOptions, headers) {
  _logger().default.debug(`scope.fetch started, path ${path}, options: ${fetchOptions}`);

  if (!fetchOptions.type) fetchOptions.type = 'component'; // for backward compatibility

  const args = _objectSpread({
    path,
    ids
  }, fetchOptions); // This might be undefined in case of fork process like during bit test command


  if (HooksManagerInstance) {
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    HooksManagerInstance.triggerHook(_constants().PRE_SEND_OBJECTS, args, headers);
  }

  const scope = await (0, _scope().loadScope)(path);
  const objectList = new (_objectList().ObjectList)();

  switch (fetchOptions.type) {
    case 'component':
      {
        const bitIds = _bitId().BitIds.deserialize(ids);

        const {
          withoutDependencies,
          includeArtifacts
        } = fetchOptions;

        const scopeComponentsImporter = _scopeComponentsImporter().default.getInstance(scope);

        const importedComponents = withoutDependencies ? await scopeComponentsImporter.fetchWithoutDeps(bitIds) : await scopeComponentsImporter.fetchWithDeps(bitIds); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

        const clientVersion = headers ? headers.version : null;
        const collectParents = !withoutDependencies;
        const componentObjects = await scopeComponentsImporter.componentsToComponentsObjects(importedComponents, clientVersion, collectParents, includeArtifacts);
        objectList.addIfNotExist(componentObjects);
        break;
      }

    case 'component-delta':
      {
        const bitIdsWithHashToStop = _bitId().BitIds.deserialize(ids);

        const scopeComponentsImporter = _scopeComponentsImporter().default.getInstance(scope);

        const bitIdsLatest = bitIdsWithHashToStop.toVersionLatest();
        const importedComponents = await scopeComponentsImporter.fetchWithoutDeps(bitIdsLatest);
        const options = {
          collectParents: true,
          collectArtifacts: fetchOptions.includeArtifacts
        };
        const clientVersion = headers ? headers.version : null;
        const allObjects = await (0, _pMapSeries().default)(importedComponents, component => {
          var _bitIdsWithHashToStop;

          const hashToStop = (_bitIdsWithHashToStop = bitIdsWithHashToStop.searchWithoutVersion(component.id)) === null || _bitIdsWithHashToStop === void 0 ? void 0 : _bitIdsWithHashToStop.version;
          options.collectParentsUntil = hashToStop ? _objects().Ref.from(hashToStop) : null;
          return component.collectObjects(scope.objects, clientVersion, options);
        });
        objectList.addIfNotExist((0, _lodash().flatten)(allObjects));
        break;
      }

    case 'lane':
      {
        const laneIds = ids.map(id => _laneId().RemoteLaneId.parse(id));
        const lanes = await scope.listLanes();
        const lanesToFetch = laneIds.map(laneId => {
          const laneToFetch = lanes.find(lane => lane.name === laneId.name); // @todo: throw LaneNotFound, make sure it shows correctly on the client using ssh

          if (!laneToFetch) throw new Error(`lane ${laneId.name} was not found in scope ${scope.name}`);
          return laneToFetch;
        });
        const lanesObjects = await Promise.all(lanesToFetch.map(async laneToFetch => {
          laneToFetch.scope = scope.name;
          const laneBuffer = await laneToFetch.compress();
          return {
            ref: laneToFetch.hash(),
            buffer: laneBuffer
          };
        }));
        objectList.addIfNotExist(lanesObjects);
        break;
      }

    case 'object':
      {
        const refs = ids.map(id => new (_objects().Ref)(id));
        const objectsItems = await scope.getObjectItems(refs);
        objectList.addIfNotExist(objectsItems);
        break;
      }

    default:
      throw new Error(`type ${fetchOptions.type} was not implemented`);
  }

  if (HooksManagerInstance) {
    await HooksManagerInstance.triggerHook(_constants().POST_SEND_OBJECTS, {
      objectList,
      scopePath: path,
      ids,
      scopeName: scope.scopeJson.name
    }, // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    headers);
  }

  _logger().default.debug('scope.fetch completed');

  return objectList;
}