/// <reference types="node" />
import type { Consumer } from '..';
import { BitId, BitIds } from '../../bit-id';
import { BitIdStr } from '../../bit-id/bit-id';
import { RemoteLaneId } from '../../lane-id/lane-id';
import { PathLinux, PathOsBased, PathOsBasedAbsolute, PathOsBasedRelative, PathRelative } from '../../utils/path';
import { ComponentFsCache } from '../component/component-fs-cache';
import ComponentMap, { ComponentMapFile, ComponentOrigin, PathChange } from './component-map';
import WorkspaceLane from './workspace-lane';
export declare type PathChangeResult = {
    id: BitId;
    changes: PathChange[];
};
export declare type IgnoreFilesDirs = {
    files: PathLinux[];
    dirs: PathLinux[];
};
export declare type GetBitMapComponentOptions = {
    ignoreVersion?: boolean;
    ignoreScopeAndVersion?: boolean;
};
export declare const LANE_KEY = "_bit_lane";
export default class BitMap {
    projectRoot: string;
    private mapPath;
    version: string;
    private isLegacy;
    workspaceLane: WorkspaceLane | null;
    private remoteLaneName?;
    components: ComponentMap[];
    hasChanged: boolean;
    paths: {
        [path: string]: BitId;
    };
    pathsLowerCase: {
        [path: string]: BitId;
    };
    markAsChangedBinded: Function;
    _cacheIds: {
        [origin: string]: BitIds | undefined;
    };
    allTrackDirs: {
        [trackDir: string]: BitId;
    } | null | undefined;
    constructor(projectRoot: string, mapPath: string, version: string, isLegacy: boolean, workspaceLane: WorkspaceLane | null, remoteLaneName?: RemoteLaneId | undefined);
    markAsChanged(): void;
    setComponent(bitId: BitId, componentMap: ComponentMap): void;
    setComponentProp(id: BitId, propName: keyof ComponentMap, val: any): ComponentMap;
    isEmpty(): any;
    removeComponentProp(id: BitId, propName: keyof ComponentMap): ComponentMap;
    static load(consumer: Consumer): Promise<BitMap>;
    loadFiles(componentFsCache: ComponentFsCache): Promise<void>;
    static loadRawSync(dirPath: PathOsBasedAbsolute): Buffer | undefined;
    static getBitMapLocation(dirPath: PathOsBasedAbsolute): {
        currentLocation: string | undefined;
        defaultLocation: string;
    };
    static reset(dirPath: PathOsBasedAbsolute, resetHard: boolean): void;
    resetToNewComponents(): void;
    private throwForDuplicateRootDirs;
    loadComponents(componentsJson: Record<string, any>): void;
    getAllComponents(origin?: ComponentOrigin | ComponentOrigin[]): ComponentMap[];
    getAllBitIdsFromAllLanes(origin?: ComponentOrigin[]): BitIds;
    getAllIdsAvailableOnLane(origin?: ComponentOrigin[]): BitIds;
    getBitId(bitId: BitId, { ignoreVersion, ignoreScopeAndVersion, }?: {
        ignoreVersion?: boolean;
        ignoreScopeAndVersion?: boolean;
    }): BitId;
    getBitIdIfExist(bitId: BitId, { ignoreVersion, ignoreScopeAndVersion, }?: {
        ignoreVersion?: boolean;
        ignoreScopeAndVersion?: boolean;
    }): BitId | undefined;
    getComponent(bitId: BitId, { ignoreVersion, ignoreScopeAndVersion }?: GetBitMapComponentOptions): ComponentMap;
    getComponentIfExist(bitId: BitId, { ignoreVersion, ignoreScopeAndVersion, }?: {
        ignoreVersion?: boolean;
        ignoreScopeAndVersion?: boolean;
    }): ComponentMap | undefined;
    getNonNestedComponentIfExist(bitId: BitId): ComponentMap | undefined;
    getComponentPreferNonNested(bitId: BitId): ComponentMap | undefined;
    getAuthoredAndImportedBitIds(): BitIds;
    getAuthoredAndImportedBitIdsOfDefaultLane(): BitIds;
    getAuthoredExportedComponents(): BitId[];
    getAuthoredNonExportedComponents(): BitId[];
    _makePathRelativeToProjectRoot(pathToChange: PathRelative): PathOsBasedRelative;
    findSimilarIds(id: BitId, compareWithoutScope?: boolean): BitIds;
    deleteOlderVersionsOfComponent(componentId: BitId): void;
    getExistingBitId(id: BitIdStr, shouldThrow?: boolean, searchWithoutScopeInProvidedId?: boolean): BitId | undefined;
    _areFilesArraysEqual(filesA: ComponentMapFile[], filesB: ComponentMapFile[]): boolean;
    mergeFilesArray(filesA: ComponentMapFile[], filesB: ComponentMapFile[]): ComponentMapFile[];
    addComponent({ componentId, files, mainFile, origin, rootDir, trackDir, originallySharedDir, wrapDir, onLanesOnly, }: {
        componentId: BitId;
        files: ComponentMapFile[];
        mainFile: PathLinux;
        origin: ComponentOrigin;
        rootDir?: PathOsBasedAbsolute | PathOsBasedRelative;
        trackDir?: PathOsBased;
        originallySharedDir?: PathLinux;
        wrapDir?: PathLinux;
        onLanesOnly?: boolean;
    }): ComponentMap;
    addFilesToComponent({ componentId, files }: {
        componentId: BitId;
        files: ComponentMapFile[];
    }): ComponentMap;
    reLoadAfterSwitchingLane(workspaceLane: null | WorkspaceLane): void;
    sortValidateAndMarkAsChanged(componentMap: ComponentMap): void;
    _invalidateCache: () => void;
    _removeFromComponentsArray(componentId: BitId): void;
    removeComponent(bitId: BitId): BitId | undefined;
    removeComponents(ids: BitIds): (BitId | undefined)[];
    isExistWithSameVersion(id: BitId): boolean;
    updateComponentId(id: BitId, updateScopeOnly?: boolean): BitId;
    updateLanesProperty(workspaceLane: WorkspaceLane, remoteLaneId: RemoteLaneId): void;
    getComponentIdByPath(componentPath: PathLinux, caseSensitive?: boolean): BitId;
    _populateAllPaths(): void;
    getAllTrackDirs(): {
        [trackDir: string]: BitId;
    };
    updatePathLocation(from: PathOsBasedRelative, to: PathOsBasedRelative, existingPath: PathOsBasedAbsolute): PathChangeResult[];
    setRemoteLane(remoteLane: RemoteLaneId): void;
    toObjects(): Record<string, any>;
    write(componentFsCache: ComponentFsCache): Promise<any>;
    private contentToString;
    getContent(): Record<string, any>;
}
