"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.promise.js");

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _chai() {
  const data = require("chai");

  _chai = function () {
    return data;
  };

  return data;
}

function path() {
  const data = _interopRequireWildcard(require("path"));

  path = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _bitMap() {
  const data = _interopRequireDefault(require("./bit-map"));

  _bitMap = function () {
    return data;
  };

  return data;
}

function _duplicateRootDir() {
  const data = require("./exceptions/duplicate-root-dir");

  _duplicateRootDir = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const scope = {
  path: path().join(__dirname, '.bit'),
  lanes: {
    getCurrentLaneName: () => 'master'
  }
};
const bitMapFixtureDir = path().join(__dirname, '../../../fixtures/bitmap-fixtures');

const getBitmapInstance = async () => {
  const consumer = {
    getPath: () => __dirname,
    isLegacy: true,
    scope
  }; // @ts-ignore

  return _bitMap().default.load(consumer);
};

const addComponentParamsFixture = {
  componentId: new (_bitId().BitId)({
    name: 'is-string'
  }),
  files: [{
    name: 'is-string.js',
    relativePath: 'is-string.js',
    test: false
  }],
  mainFile: 'is-string.js',
  origin: _constants().COMPONENT_ORIGINS.AUTHORED
};
const addComponentParamsImportedFixture = {
  componentId: new (_bitId().BitId)({
    scope: 'my-scope',
    name: 'is-string-imported',
    version: '0.0.1'
  }),
  files: [{
    name: 'is-string-imported.js',
    relativePath: 'is-string-imported.js',
    test: false
  }],
  mainFile: 'is-string-imported.js',
  origin: _constants().COMPONENT_ORIGINS.IMPORTED,
  rootDir: 'utils'
};
describe('BitMap', function () {
  // @ts-ignore
  _logger().default.debug = () => {}; // @ts-ignore


  _logger().default.info = () => {};

  this.timeout(0);
  describe('toObject', () => {
    let bitMap;
    let componentMap;
    before(async () => {
      bitMap = await getBitmapInstance();
      bitMap.addComponent(addComponentParamsFixture);
      const allComponents = bitMap.toObjects();
      componentMap = allComponents['is-string'];
    });
    it('should remove the "id" property', () => {
      (0, _chai().expect)(componentMap).to.not.have.property('id');
    });
    it('should add "exported" property to authored components', () => {
      (0, _chai().expect)(componentMap).to.have.property('exported');
      (0, _chai().expect)(componentMap.exported).to.be.false;
    });
    it('should not add "exported" property to imported components', () => {
      bitMap.addComponent(addComponentParamsImportedFixture);
      const allComponents = bitMap.toObjects();
      const componentMapImported = allComponents['my-scope/is-string-imported@0.0.1'];
      (0, _chai().expect)(componentMapImported).to.not.have.property('exported');
    });
    it('should sort the components alphabetically', async () => {
      const exampleComponent = _objectSpread({}, addComponentParamsFixture);

      bitMap = await getBitmapInstance();
      exampleComponent.componentId = new (_bitId().BitId)({
        scope: 'my-scope',
        name: 'is-string1',
        version: '0.0.1'
      });
      bitMap.addComponent(exampleComponent);
      exampleComponent.componentId = new (_bitId().BitId)({
        scope: 'my-scope',
        name: 'is-string3',
        version: '0.0.1'
      });
      bitMap.addComponent(exampleComponent);
      exampleComponent.componentId = new (_bitId().BitId)({
        scope: 'my-scope',
        name: 'is-string2',
        version: '0.0.1'
      });
      bitMap.addComponent(exampleComponent);
      const allComponents = bitMap.toObjects();
      const ids = Object.keys(allComponents);
      (0, _chai().expect)(ids[0]).to.equal('my-scope/is-string1@0.0.1');
      (0, _chai().expect)(ids[1]).to.equal('my-scope/is-string2@0.0.1');
      (0, _chai().expect)(ids[2]).to.equal('my-scope/is-string3@0.0.1');
    });
    it('should sort the files in the component alphabetically', async () => {
      const exampleComponent = _objectSpread({}, addComponentParamsFixture);

      bitMap = await getBitmapInstance();
      exampleComponent.files = [{
        name: 'is-string1.js',
        relativePath: 'is-string1.js',
        test: false
      }, {
        name: 'is-string3.js',
        relativePath: 'is-string3.js',
        test: false
      }, {
        name: 'is-string2.js',
        relativePath: 'is-string2.js',
        test: false
      }];
      exampleComponent.mainFile = 'is-string2.js';
      bitMap.addComponent(exampleComponent);
      const allComponents = bitMap.toObjects();
      const files = allComponents['is-string'].files;
      (0, _chai().expect)(files[0].relativePath).to.equal('is-string1.js');
      (0, _chai().expect)(files[1].relativePath).to.equal('is-string2.js');
      (0, _chai().expect)(files[2].relativePath).to.equal('is-string3.js');
    });
    it('should sort the fields in the component files alphabetically', async () => {
      const exampleComponent = _objectSpread({}, addComponentParamsFixture);

      bitMap = await getBitmapInstance();
      bitMap.addComponent(exampleComponent);
      const allComponents = bitMap.toObjects();
      const files = allComponents['is-string'].files;
      const fields = Object.keys(files[0]);
      (0, _chai().expect)(fields[0]).to.equal('name');
      (0, _chai().expect)(fields[1]).to.equal('relativePath');
      (0, _chai().expect)(fields[2]).to.equal('test');
    });
  });
  describe('loadComponents', () => {
    let bitMap;
    before(async () => {
      bitMap = await getBitmapInstance();
    });
    it('should throw DuplicateRootDir error when multiple ids have the same rootDir', () => {
      const invalidBitMap = {
        comp1: {
          mainFile: 'index.js',
          rootDir: 'comp1'
        },
        comp2: {
          mainFile: 'index.js',
          rootDir: 'comp1'
        }
      };
      (0, _chai().expect)(() => bitMap.loadComponents(invalidBitMap)).to.throw(_duplicateRootDir().DuplicateRootDir);
    });
  });
  describe('getAuthoredExportedComponents', async () => {
    it('should return an empty array when there are no authored components', async () => {
      const consumer = {
        getPath: () => path().join(bitMapFixtureDir, 'only-imported'),
        isLegacy: true,
        scope
      }; // @ts-ignore

      const bitMap = await _bitMap().default.load(consumer);
      const results = bitMap.getAuthoredExportedComponents();
      (0, _chai().expect)(results).to.be.an('array');
      (0, _chai().expect)(results).to.have.lengthOf(0);
    });
  });
});