"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/es.string.replace.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _generalError() {
  const data = _interopRequireDefault(require("../../error/general-error"));

  _generalError = function () {
    return data;
  };

  return data;
}

function _showDoctorError() {
  const data = _interopRequireDefault(require("../../error/show-doctor-error"));

  _showDoctorError = function () {
    return data;
  };

  return data;
}

function _repositories() {
  const data = require("../../scope/repositories");

  _repositories = function () {
    return data;
  };

  return data;
}

function _diffFiles() {
  const data = _interopRequireDefault(require("../../utils/diff-files"));

  _diffFiles = function () {
    return data;
  };

  return data;
}

function _componentsObjectDiff() {
  const data = require("./components-object-diff");

  _componentsObjectDiff = function () {
    return data;
  };

  return data;
}

var componentsDiff = async function componentsDiff(consumer, ids, version, toVersion, verbose) {
  // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
  const {
    components
  } = await consumer.loadComponents(ids);
  if (!components) throw new (_showDoctorError().default)('failed loading the components');
  const tmp = new (_repositories().Tmp)(consumer.scope); // try to resolve ids scope of by components array

  const idsWithScope = ids.map(id => {
    if (!id.scope && components) {
      const foundComponent = components.find(o => o.name === id.name);
      if (foundComponent) return id.changeScope(foundComponent.scope);
    }

    return id;
  });

  try {
    const getResults = () => {
      if (version && toVersion) {
        return Promise.all(idsWithScope.map(id => getComponentDiffBetweenVersions(id)));
      }

      if (version) {
        return Promise.all(components.map(component => getComponentDiffOfVersion(component)));
      }

      return Promise.all(components.map(component => getComponentDiff(component)));
    };

    const componentsDiffResults = await getResults();
    await tmp.clear();
    return componentsDiffResults;
  } catch (err) {
    await tmp.clear();
    throw err;
  }

  async function getComponentDiffOfVersion(component) {
    const diffResult = {
      id: component.id,
      hasDiff: false
    };
    const modelComponent = await consumer.scope.getModelComponentIfExist(component.id);

    if (!modelComponent) {
      throw new (_generalError().default)(`component ${component.id.toString()} doesn't have any version yet`);
    }

    const repository = consumer.scope.objects; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const fromVersionObject = await modelComponent.loadVersion(version, repository);
    const versionFiles = await fromVersionObject.modelFilesToSourceFiles(repository);
    const fsFiles = component.files; // $FlowFixMe version must be defined as the component.componentFromModel do exist
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const versionB = component.id.version; // $FlowFixMe this function gets called only when version is set
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    diffResult.filesDiff = await getFilesDiff(tmp, versionFiles, fsFiles, version, versionB); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const fromVersionComponent = await modelComponent.toConsumerComponent(version, consumer.scope.name, repository);
    await updateFieldsDiff(fromVersionComponent, component, diffResult);
    return diffResult;
  }

  async function getComponentDiffBetweenVersions(id) {
    const diffResult = {
      id,
      hasDiff: false
    };
    const modelComponent = await consumer.scope.getModelComponentIfExist(id);

    if (!modelComponent) {
      throw new (_generalError().default)(`component ${id.toString()} doesn't have any version yet`);
    }

    const repository = consumer.scope.objects; // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const fromVersionObject = await modelComponent.loadVersion(version, repository); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const toVersionObject = await modelComponent.loadVersion(toVersion, repository);
    const fromVersionFiles = await fromVersionObject.modelFilesToSourceFiles(repository);
    const toVersionFiles = await toVersionObject.modelFilesToSourceFiles(repository); // $FlowFixMe version and toVersion are set when calling this function
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    diffResult.filesDiff = await getFilesDiff(tmp, fromVersionFiles, toVersionFiles, version, toVersion); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const fromVersionComponent = await modelComponent.toConsumerComponent(version, consumer.scope.name, repository); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const toVersionComponent = await modelComponent.toConsumerComponent(toVersion, consumer.scope.name, repository);
    await updateFieldsDiff(fromVersionComponent, toVersionComponent, diffResult);
    return diffResult;
  }

  async function getComponentDiff(component) {
    const diffResult = {
      id: component.id,
      hasDiff: false
    };

    if (!component.componentFromModel) {
      // it's a new component. not modified. nothing to check.
      return diffResult;
    }

    const modelFiles = component.componentFromModel.files;
    const fsFiles = component.files; // $FlowFixMe version must be defined as the component.componentFromModel do exist
    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    diffResult.filesDiff = await getFilesDiff(tmp, modelFiles, fsFiles, component.id.version, component.id.version); // $FlowFixMe we made sure already that component.componentFromModel is defined

    await updateFieldsDiff(component.componentFromModel, component, diffResult);
    return diffResult;
  }

  async function updateFieldsDiff(componentA, componentB, diffResult) {
    diffResult.fieldsDiff = (0, _componentsObjectDiff().diffBetweenComponentsObjects)(consumer, componentA, componentB, verbose);
    diffResult.hasDiff = hasDiff(diffResult);
  }
};

exports.default = componentsDiff;

function hasDiff(diffResult) {
  return !!(diffResult.filesDiff && diffResult.filesDiff.find(file => file.diffOutput) || diffResult.fieldsDiff);
}

async function getOneFileDiff(filePathA, filePathB, fileALabel, fileBLabel, fileName) {
  const fileDiff = await (0, _diffFiles().default)(filePathA, filePathB);
  if (!fileDiff) return '';
  const diffStartsString = '--- '; // the part before this string is not needed for our purpose

  const diffStart = fileDiff.indexOf(diffStartsString);
  if (!diffStart || diffStart < 1) return ''; // invalid diff
  // e.g. Linux: --- a/private/var/folders/z ... .js
  // Windows: --- "a/C:\\Users\\David\\AppData\\Local\\Temp\\bit ... .js

  const regExpA = /--- ["]?a.*\n/; // exact "---", follow by a or "a (for Windows) then \n

  const regExpB = /\+\+\+ ["]?b.*\n/; // exact "+++", follow by b or "b (for Windows) then \n

  return fileDiff.substr(diffStart).replace(regExpA, `--- ${fileName} (${fileALabel})\n`).replace(regExpB, `+++ ${fileName} (${fileBLabel})\n`);
}

async function getFilesDiff(tmp, filesA, filesB, filesAVersion, filesBVersion, // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
fileNameAttribute = 'relative') {
  const filesAPaths = filesA.map(f => f[fileNameAttribute]);
  const filesBPaths = filesB.map(f => f[fileNameAttribute]);

  const allPaths = _ramda().default.uniq(filesAPaths.concat(filesBPaths));

  const fileALabel = filesAVersion === filesBVersion ? `${filesAVersion} original` : filesAVersion;
  const fileBLabel = filesAVersion === filesBVersion ? `${filesBVersion} modified` : filesBVersion;
  const filesDiffP = allPaths.map(async relativePath => {
    const getFilePath = async files => {
      const file = files.find(f => f[fileNameAttribute] === relativePath);
      const fileContent = file ? file.contents : '';
      return tmp.save(fileContent);
    };

    const [fileAPath, fileBPath] = await Promise.all([getFilePath(filesA), getFilePath(filesB)]);
    const diffOutput = await getOneFileDiff(fileAPath, fileBPath, fileALabel, fileBLabel, relativePath);
    return {
      filePath: relativePath,
      diffOutput
    };
  });
  return Promise.all(filesDiffP);
}