"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DependenciesLoader = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _lastModified() {
  const data = require("../../../../utils/fs/last-modified");

  _lastModified = function () {
    return data;
  };

  return data;
}

function _config() {
  const data = require("../../../config");

  _config = function () {
    return data;
  };

  return data;
}

function _dependenciesData() {
  const data = require("./dependencies-data");

  _dependenciesData = function () {
    return data;
  };

  return data;
}

function _dependenciesResolver() {
  const data = _interopRequireDefault(require("./dependencies-resolver"));

  _dependenciesResolver = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

class DependenciesLoader {
  constructor(component, consumer, opts) {
    this.component = component;
    this.consumer = consumer;
    this.opts = opts;
    (0, _defineProperty2().default)(this, "idStr", void 0);
    this.idStr = this.component.id.toString();
  }

  async load() {
    const dependenciesData = await this.getDependenciesData();
    this.setDependenciesDataOnComponent(dependenciesData);
  }

  async getDependenciesData() {
    const depsDataFromCache = await this.getDependenciesDataFromCacheIfPossible();

    if (depsDataFromCache) {
      return depsDataFromCache;
    }

    const dependencyResolver = new (_dependenciesResolver().default)(this.component, this.consumer);
    const dependenciesData = await dependencyResolver.getDependenciesData(this.opts.cacheResolvedDependencies, this.opts.cacheProjectAst);

    if (this.shouldSaveInCache(dependenciesData)) {
      await this.consumer.componentFsCache.saveDependenciesDataInCache(this.idStr, dependenciesData.serialize());
    }

    return dependenciesData;
  }

  async getDependenciesDataFromCacheIfPossible() {
    var _this$component$compo;

    if (!this.opts.useDependenciesCache) {
      return null;
    }

    const cacheData = await this.consumer.componentFsCache.getDependenciesDataFromCache(this.idStr);

    if (!cacheData) {
      return null; // probably the first time, so it wasn't entered to the cache yet.
    }

    const rootDir = (_this$component$compo = this.component.componentMap) === null || _this$component$compo === void 0 ? void 0 : _this$component$compo.getComponentDir();

    if (!rootDir) {
      // could happen on legacy only and when there is no trackDir, in which case, we can't
      // determine whether or not a component file has been deleted, as a result, we are unable
      // to invalidate the cache in such a case.
      return null;
    }

    const filesPaths = this.component.files.map(f => f.path);
    const componentConfigFilename = this.consumer.isLegacy ? _constants().PACKAGE_JSON : _constants().COMPONENT_CONFIG_FILE_NAME;

    const componentConfigPath = _path().default.join(this.consumer.getPath(), rootDir, componentConfigFilename);

    filesPaths.push(componentConfigPath);
    const lastModifiedComponent = await (0, _lastModified().getLastModifiedComponentTimestampMs)(rootDir, filesPaths);
    const wasModifiedAfterCache = lastModifiedComponent > cacheData.timestamp;

    if (wasModifiedAfterCache) {
      return null; // cache is invalid.
    }

    _logger().default.debug(`dependencies-loader, getting the dependencies data for ${this.idStr} from the cache`);

    return _dependenciesData().DependenciesData.deserialize(cacheData.data);
  }

  shouldSaveInCache(dependenciesData) {
    var _dependenciesData$iss, _dependenciesData$iss2;

    if ((_dependenciesData$iss = dependenciesData.issues) !== null && _dependenciesData$iss !== void 0 && _dependenciesData$iss.missingPackagesDependenciesOnFs || (_dependenciesData$iss2 = dependenciesData.issues) !== null && _dependenciesData$iss2 !== void 0 && _dependenciesData$iss2.untrackedDependencies) {
      return false;
    }

    return true;
  }

  setDependenciesDataOnComponent(dependenciesData) {
    this.component.setDependencies(dependenciesData.allDependencies.dependencies);
    this.component.setDevDependencies(dependenciesData.allDependencies.devDependencies);
    this.component.packageDependencies = dependenciesData.allPackagesDependencies.packageDependencies;
    this.component.devPackageDependencies = dependenciesData.allPackagesDependencies.devPackageDependencies;
    this.component.peerPackageDependencies = dependenciesData.allPackagesDependencies.peerPackageDependencies;

    if (!_ramda().default.isEmpty(dependenciesData.overridesDependencies.missingPackageDependencies)) {
      dependenciesData.issues.missingPackagesDependenciesFromOverrides = dependenciesData.overridesDependencies.missingPackageDependencies;
    }

    if (!_ramda().default.isEmpty(dependenciesData.issues)) this.component.issues = dependenciesData.issues;
    this.component.manuallyRemovedDependencies = dependenciesData.overridesDependencies.manuallyRemovedDependencies;
    this.component.manuallyAddedDependencies = dependenciesData.overridesDependencies.manuallyAddedDependencies;

    if (dependenciesData.coreAspects.length) {
      this.pushToDependencyResolverExtension('coreAspects', dependenciesData.coreAspects, 'set');
    }
  }

  pushToDependencyResolverExtension(dataField, data, operation = 'add') {
    const depResolverAspectName = _dependenciesResolver().default.getDepResolverAspectName();

    if (!depResolverAspectName) return;
    let extExistOnComponent = true;
    let ext = this.component.extensions.findCoreExtension(depResolverAspectName);

    if (!ext) {
      extExistOnComponent = false; // Create new deps resolver extension entry to add to the component with data only

      ext = new (_config().ExtensionDataEntry)(undefined, undefined, depResolverAspectName, undefined, {});
    }

    if (!ext.data[dataField]) ext.data[dataField] = [];

    if (operation === 'add') {
      const existing = ext.data[dataField].find(c => c.packageName === data.packageName);

      if (existing) {
        existing.componentId = data.componentId;
      } else {
        ext.data[dataField].push(data);
      }
    }

    if (operation === 'set') {
      ext.data[dataField] = data;
    }

    if (!extExistOnComponent) {
      this.component.extensions.push(ext);
    }
  }

}

exports.DependenciesLoader = DependenciesLoader;