"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _pMapSeries() {
  const data = _interopRequireDefault(require("p-map-series"));

  _pMapSeries = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _models() {
  const data = require("../models");

  _models = function () {
    return data;
  };

  return data;
}

function _removedComponents() {
  const data = _interopRequireDefault(require("../removed-components"));

  _removedComponents = function () {
    return data;
  };

  return data;
}

class RemoveModelComponents {
  constructor(scope, bitIds, force, removeSameOrigin, consumer) {
    (0, _defineProperty2().default)(this, "scope", void 0);
    (0, _defineProperty2().default)(this, "bitIds", void 0);
    (0, _defineProperty2().default)(this, "force", void 0);
    (0, _defineProperty2().default)(this, "removeSameOrigin", false);
    (0, _defineProperty2().default)(this, "consumer", void 0);
    (0, _defineProperty2().default)(this, "currentLane", null);
    this.scope = scope;
    this.bitIds = bitIds;
    this.force = force;
    this.removeSameOrigin = removeSameOrigin;
    this.consumer = consumer;
  }

  async setCurrentLane() {
    this.currentLane = await this.scope.lanes.getCurrentLaneObject();
  }

  async remove() {
    const {
      missingComponents,
      foundComponents
    } = await this.scope.filterFoundAndMissingComponents(this.bitIds);
    await this.setCurrentLane();
    const dependentBits = await this.scope.findDependentBits(foundComponents);

    if (_ramda().default.isEmpty(dependentBits) || this.force) {
      // do not run this in parallel (promise.all), otherwise, it may throw an error when
      // trying to delete the same file at the same time (happens when removing a component with
      // a dependency and the dependency itself)
      const removedComponents = await (0, _pMapSeries().default)(foundComponents, bitId => this._removeSingle(bitId));
      if (this.currentLane) await this.scope.lanes.saveLane(this.currentLane);
      await this.scope.objects.persist(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      const ids = new (_bitId().BitIds)(...removedComponents.map(x => x.bitId)); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

      const removedDependencies = new (_bitId().BitIds)(..._ramda().default.flatten(removedComponents.map(x => x.removedDependencies)));
      const removedFromLane = Boolean(this.currentLane);
      return new (_removedComponents().default)({
        removedComponentIds: ids,
        missingComponents,
        removedDependencies,
        removedFromLane
      });
    } // some of the components have dependents, don't remove them


    return new (_removedComponents().default)({
      missingComponents,
      dependentBits
    });
  }
  /**
   * removeSingle - remove single component
   * @param {BitId} bitId - list of remote component ids to delete
   * @param {boolean} removeSameOrigin - remove component dependencies from same origin
   */


  async _removeSingle(bitId) {
    _logger().default.debug(`scope.removeSingle ${bitId.toString()}, remove dependencies: ${this.removeSameOrigin.toString()}`); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


    const component = (await this.scope.getModelComponentIfExist(bitId)).toComponentVersion(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const consumerComponentToRemove = await component.toConsumer(this.scope.objects);
    const componentList = await this.scope.listIncludesSymlinks();
    const dependentBits = await this.scope.findDependentBits(consumerComponentToRemove.flattenedDependencies, bitId.version !== _constants().LATEST_BIT_VERSION);
    const removedDependencies = await this._removeComponentsDependencies(dependentBits, componentList, consumerComponentToRemove, bitId);
    await this._removeComponent(bitId, componentList);
    const version = Object.keys(component.component.versions).length <= 1 ? _constants().LATEST_BIT_VERSION : bitId.version;
    return {
      bitId: bitId.changeVersion(version),
      removedDependencies
    };
  }

  async _removeComponentsDependencies(dependentBits, componentList, consumerComponentToRemove, bitId) {
    const removedComponents = consumerComponentToRemove.flattenedDependencies.map(async dependencyId => {
      const dependentsIds = dependentBits[dependencyId.toStringWithoutVersion()];

      const relevantDependents = _ramda().default.reject(dependent => dependent.isEqual(bitId) || dependent.scope !== dependencyId.scope, dependentsIds);

      let isNested = true;

      if (this.consumer) {
        const componentMapIgnoreVersion = this.consumer.bitMap.getComponentIfExist(dependencyId, {
          ignoreVersion: true
        });
        const componentMapExact = this.consumer.bitMap.getComponentIfExist(dependencyId);
        const componentMap = componentMapExact || componentMapIgnoreVersion;

        if (componentMap) {
          isNested = componentMap.origin === _constants().COMPONENT_ORIGINS.NESTED;
        }
      }

      if (_ramda().default.isEmpty(relevantDependents) && !this.bitIds.searchWithoutVersion(dependencyId) && ( // don't delete dependency if it is already deleted as an individual
      dependencyId.scope !== bitId.scope || this.removeSameOrigin) && isNested) {
        await this._removeComponent(dependencyId, componentList);
        return dependencyId;
      }

      return null;
    });
    let removedDependencies = await Promise.all(removedComponents);
    removedDependencies = removedDependencies.filter(x => !_ramda().default.isNil(x)); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    return _bitId().BitIds.fromArray(removedDependencies);
  }

  async _removeComponent(id, componentList) {
    if (this.currentLane) {
      const result = this.currentLane.removeComponent(id);
      if (!result) throw new Error(`failed deleting ${id.toString()}, the component was not found on the lane`);
      return;
    }

    const symlink = componentList.filter(component => component instanceof _models().Symlink && id.isEqualWithoutScopeAndVersion(component.toBitId()));
    await this.scope.sources.removeComponentById(id);
    if (!_ramda().default.isEmpty(symlink)) this.scope.objects.removeObject(symlink[0].hash());
  }

}

exports.default = RemoveModelComponents;