"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/es.string.replace.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerUpdateDependenciesOnTag = registerUpdateDependenciesOnTag;
exports.default = tagModelComponent;
exports.addFlattenedDependenciesToComponents = addFlattenedDependenciesToComponents;
exports.updateComponentsByTagResult = updateComponentsByTagResult;
exports.getPublishedPackages = getPublishedPackages;

function _pMapSeries() {
  const data = _interopRequireDefault(require("p-map-series"));

  _pMapSeries = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function RA() {
  const data = _interopRequireWildcard(require("ramda-adjunct"));

  RA = function () {
    return data;
  };

  return data;
}

function _uuid() {
  const data = require("uuid");

  _uuid = function () {
    return data;
  };

  return data;
}

function globalConfig() {
  const data = _interopRequireWildcard(require("../../api/consumer/lib/global-config"));

  globalConfig = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("../../cli/loader"));

  _loader = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _componentSchema() {
  const data = require("../../consumer/component/component-schema");

  _componentSchema = function () {
    return data;
  };

  return data;
}

function _exceptions() {
  const data = require("../../consumer/exceptions");

  _exceptions = function () {
    return data;
  };

  return data;
}

function _showDoctorError() {
  const data = _interopRequireDefault(require("../../error/show-doctor-error"));

  _showDoctorError = function () {
    return data;
  };

  return data;
}

function _validationError() {
  const data = _interopRequireDefault(require("../../error/validation-error"));

  _validationError = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _autoTag() {
  const data = require("./auto-tag");

  _autoTag = function () {
    return data;
  };

  return data;
}

function _getFlattenedDependencies() {
  const data = require("./get-flattened-dependencies");

  _getFlattenedDependencies = function () {
    return data;
  };

  return data;
}

function _semverHelper() {
  const data = require("../../utils/semver-helper");

  _semverHelper = function () {
    return data;
  };

  return data;
}

let updateDependenciesOnTag;

function registerUpdateDependenciesOnTag(func) {
  updateDependenciesOnTag = func;
}

function updateDependenciesVersions(componentsToTag) {
  const getNewDependencyVersion = id => {
    const foundDependency = componentsToTag.find(component => component.id.isEqualWithoutVersion(id));
    return foundDependency ? id.changeVersion(foundDependency.version) : null;
  };

  const changeExtensionsVersion = component => {
    component.extensions.forEach(ext => {
      if (ext.extensionId) {
        const newDepId = getNewDependencyVersion(ext.extensionId);
        if (newDepId) ext.extensionId = newDepId;
      }
    });
  };

  componentsToTag.forEach(oneComponentToTag => {
    oneComponentToTag.getAllDependencies().forEach(dependency => {
      const newDepId = getNewDependencyVersion(dependency.id);
      if (newDepId) dependency.id = newDepId;
    });
    changeExtensionsVersion(oneComponentToTag);

    if (updateDependenciesOnTag && typeof updateDependenciesOnTag === 'function') {
      // @ts-ignore
      oneComponentToTag = updateDependenciesOnTag(oneComponentToTag, getNewDependencyVersion.bind(this));
    }
  });
}

function setHashes(componentsToTag) {
  componentsToTag.forEach(componentToTag => {
    componentToTag.version = (0, _utils().sha1)((0, _uuid().v4)());
  });
}

async function setFutureVersions(componentsToTag, scope, releaseType, exactVersion, persist) {
  await Promise.all(componentsToTag.map(async componentToTag => {
    var _componentToTag$compo, _componentToTag$compo2;

    // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
    const modelComponent = await scope.sources.findOrAddComponent(componentToTag);
    const nextVersion = (_componentToTag$compo = componentToTag.componentMap) === null || _componentToTag$compo === void 0 ? void 0 : (_componentToTag$compo2 = _componentToTag$compo.nextVersion) === null || _componentToTag$compo2 === void 0 ? void 0 : _componentToTag$compo2.version;

    if (nextVersion && persist) {
      const exactVersionOrReleaseType = (0, _semverHelper().getValidVersionOrReleaseType)(nextVersion);
      if (exactVersionOrReleaseType.exactVersion) exactVersion = exactVersionOrReleaseType.exactVersion;
      if (exactVersionOrReleaseType.releaseType) releaseType = exactVersionOrReleaseType.releaseType;
    }

    const version = modelComponent.getVersionToAdd(releaseType, exactVersion);
    componentToTag.previouslyUsedVersion = componentToTag.version;
    componentToTag.version = version;
  }));
}
/**
 * make sure the originallySharedDir was added before saving the component. also, make sure it was
 * not added twice.
 * we need three objects for this:
 * 1) component.pendingVersion => version pending to be saved in the filesystem. we want to make sure it has the added sharedDir.
 * 2) component.componentFromModel => previous version of the component. it has the original sharedDir.
 * 3) component.componentMap => current paths in the filesystem, which don't have the sharedDir.
 *
 * The component may be changed from the componentFromModel. The files may be removed and added and
 * new files may added, so we can't compare the files of componentFromModel to component.
 *
 * What we can do is calculating the sharedDir from component.componentFromModel
 * then, make sure that calculatedSharedDir + pathFromComponentMap === component.pendingVersion
 *
 * Also, make sure that the wrapDir has been removed
 */


function validateDirManipulation(components) {
  const throwOnError = (expectedPath, actualPath) => {
    if (expectedPath !== actualPath) {
      throw new (_validationError().default)(`failed validating the component paths with sharedDir, expected path ${expectedPath}, got ${actualPath}`);
    }
  };

  const validateComponent = component => {
    if (!component.componentMap) throw new Error(`componentMap is missing from ${component.id.toString()}`);
    if (!component.componentFromModel) return; // component.componentFromModel.setOriginallySharedDir();

    const sharedDir = component.componentFromModel.originallySharedDir;
    const wrapDir = component.componentFromModel.wrapDir;

    const pathWithSharedDir = pathStr => {
      // $FlowFixMe componentMap is set here
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      if (sharedDir && component.componentMap.origin === _constants().COMPONENT_ORIGINS.IMPORTED) {
        return (0, _utils().pathJoinLinux)(sharedDir, pathStr);
      }

      return pathStr;
    };

    const pathWithoutWrapDir = pathStr => {
      if (wrapDir) {
        return pathStr.replace(`${wrapDir}/`, '');
      }

      return pathStr;
    };

    const pathAfterDirManipulation = pathStr => {
      const withoutWrapDir = pathWithoutWrapDir(pathStr);
      return pathWithSharedDir(withoutWrapDir);
    };

    const expectedMainFile = pathAfterDirManipulation(component.componentMap.mainFile); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    throwOnError(expectedMainFile, component.pendingVersion.mainFile); // $FlowFixMe componentMap is set here

    const componentMapFiles = component.componentMap.getAllFilesPaths(); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!

    const componentFiles = component.pendingVersion.files.map(file => file.relativePath);
    componentMapFiles.forEach(file => {
      const expectedFile = pathAfterDirManipulation(file);

      if (!componentFiles.includes(expectedFile)) {
        throw new (_validationError().default)(`failed validating the component paths, expected a file ${expectedFile} to be in ${componentFiles.toString()} array`);
      }
    });
  };

  components.forEach(component => validateComponent(component));
}

async function tagModelComponent({
  consumerComponents,
  scope,
  message,
  exactVersion,
  releaseType,
  force,
  consumer,
  ignoreNewestVersion = false,
  skipTests = false,
  verbose = false,
  skipAutoTag,
  soft,
  build,
  persist,
  resolveUnmerged,
  isSnap = false,
  disableDeployPipeline
}) {
  if (!persist) skipTests = true;
  const consumerComponentsIdsMap = {}; // Concat and unique all the dependencies from all the components so we will not import
  // the same dependency more then once, it's mainly for performance purpose

  consumerComponents.forEach(consumerComponent => {
    const componentIdString = consumerComponent.id.toString(); // Store it in a map so we can take it easily from the sorted array which contain only the id

    consumerComponentsIdsMap[componentIdString] = consumerComponent;
  });

  const componentsToTag = _ramda().default.values(consumerComponentsIdsMap); // consumerComponents unique


  const idsToTag = _bitId().BitIds.fromArray(componentsToTag.map(c => c.id)); // ids without versions are new. it's impossible that tagged (and not-modified) components has
  // them as dependencies.


  const idsToTriggerAutoTag = idsToTag.filter(id => id.hasVersion());
  const autoTagData = skipAutoTag ? [] : await (0, _autoTag().getAutoTagInfo)(consumer, _bitId().BitIds.fromArray(idsToTriggerAutoTag));
  const autoTagComponents = autoTagData.map(autoTagItem => autoTagItem.component);
  const autoTagComponentsFiltered = autoTagComponents.filter(c => !idsToTag.has(c.id));
  const allComponentsToTag = [...componentsToTag, ...autoTagComponentsFiltered]; // check for each one of the components whether it is using an old version

  if (!ignoreNewestVersion && !isSnap) {
    const newestVersionsP = allComponentsToTag.map(async component => {
      if (component.componentFromModel) {
        // otherwise it's a new component, so this check is irrelevant
        const modelComponent = await scope.getModelComponentIfExist(component.id);
        if (!modelComponent) throw new (_showDoctorError().default)(`component ${component.id} was not found in the model`);
        if (!modelComponent.listVersions().length) return null; // no versions yet, no issues.

        const latest = modelComponent.latest();

        if (latest !== component.version) {
          return {
            componentId: component.id.toStringWithoutVersion(),
            currentVersion: component.version,
            latestVersion: latest
          };
        }
      }

      return null;
    });
    const newestVersions = await Promise.all(newestVersionsP);
    const newestVersionsWithoutEmpty = newestVersions.filter(newest => newest);

    if (!RA().isNilOrEmpty(newestVersionsWithoutEmpty)) {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      throw new (_exceptions().NewerVersionFound)(newestVersionsWithoutEmpty);
    }
  }

  let testsResults = [];

  if (consumer.isLegacy) {
    _logger().default.debugAndAddBreadCrumb('tag-model-components', 'sequentially build all components');

    await scope.buildMultiple(allComponentsToTag, consumer, false, verbose);

    _logger().default.debug('scope.putMany: sequentially test all components');

    if (!skipTests) {
      const testsResultsP = scope.testMultiple({
        components: allComponentsToTag,
        consumer,
        verbose,
        rejectOnFailure: !force
      });

      try {
        // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
        testsResults = await testsResultsP;
      } catch (err) {
        // if force is true, ignore the tests and continue
        if (!force) {
          // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
          if (!verbose) throw new (_exceptions().ComponentSpecsFailed)();
          throw err;
        }
      }
    }
  }

  _logger().default.debugAndAddBreadCrumb('tag-model-components', 'sequentially persist all components'); // go through all components and find the future versions for them


  isSnap ? setHashes(allComponentsToTag) : await setFutureVersions(allComponentsToTag, scope, releaseType, exactVersion, persist);
  setCurrentSchema(allComponentsToTag, consumer); // go through all dependencies and update their versions

  updateDependenciesVersions(allComponentsToTag);
  await addLogToComponents(componentsToTag, autoTagComponents, persist, message);

  if (soft) {
    consumer.updateNextVersionOnBitmap(allComponentsToTag, exactVersion, releaseType);
  } else {
    if (!skipTests) addSpecsResultsToComponents(allComponentsToTag, testsResults);
    await addFlattenedDependenciesToComponents(consumer.scope, allComponentsToTag);
    emptyBuilderData(allComponentsToTag);
    addBuildStatus(consumer, allComponentsToTag, _constants().BuildStatus.Pending);
    await addComponentsToScope(consumer, allComponentsToTag, Boolean(resolveUnmerged));
    validateDirManipulation(allComponentsToTag);
    await consumer.updateComponentsVersions(allComponentsToTag);
  }

  const publishedPackages = [];

  if (!consumer.isLegacy && build) {
    const onTagOpts = {
      disableDeployPipeline,
      throwOnError: true
    };
    const results = await (0, _pMapSeries().default)(scope.onTag, func => func(allComponentsToTag, onTagOpts));
    results.forEach(tagResult => updateComponentsByTagResult(allComponentsToTag, tagResult));
    publishedPackages.push(...getPublishedPackages(allComponentsToTag));
    addBuildStatus(consumer, allComponentsToTag, _constants().BuildStatus.Succeed);
    await (0, _pMapSeries().default)(allComponentsToTag, consumerComponent => scope.sources.enrichSource(consumerComponent));
  }

  if (!soft) {
    await scope.objects.persist();
  }

  return {
    taggedComponents: componentsToTag,
    autoTaggedResults: autoTagData,
    publishedPackages
  };
}

async function addComponentsToScope(consumer, components, resolveUnmerged) {
  const lane = await consumer.getCurrentLaneObject();
  await (0, _pMapSeries().default)(components, async component => {
    await consumer.scope.sources.addSource({
      source: component,
      consumer,
      lane,
      resolveUnmerged
    });
  });
}

function emptyBuilderData(components) {
  components.forEach(component => {
    const existingBuilder = component.extensions.findCoreExtension(_constants().Extensions.builder);
    if (existingBuilder) existingBuilder.data = {};
  });
}

async function addFlattenedDependenciesToComponents(scope, components) {
  _loader().default.start('importing missing dependencies...');

  const flattenedDependenciesGetter = new (_getFlattenedDependencies().FlattenedDependenciesGetter)(scope, components);
  await flattenedDependenciesGetter.populateFlattenedDependencies();

  _loader().default.stop();
}

function addSpecsResultsToComponents(components, testsResults) {
  components.forEach(component => {
    const testResult = testsResults.find(result => {
      // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
      return component.id.isEqualWithoutScopeAndVersion(result.componentId);
    });
    component.specsResults = testResult ? testResult.specs : undefined;
  });
}

async function addLogToComponents(components, autoTagComps, persist, message) {
  const username = await globalConfig().get(_constants().CFG_USER_NAME_KEY);
  const email = await globalConfig().get(_constants().CFG_USER_EMAIL_KEY);

  const getLog = component => {
    var _component$componentM;

    const nextVersion = persist ? (_component$componentM = component.componentMap) === null || _component$componentM === void 0 ? void 0 : _component$componentM.nextVersion : null;
    return {
      username: (nextVersion === null || nextVersion === void 0 ? void 0 : nextVersion.username) || username,
      email: (nextVersion === null || nextVersion === void 0 ? void 0 : nextVersion.email) || email,
      message: (nextVersion === null || nextVersion === void 0 ? void 0 : nextVersion.message) || message,
      date: Date.now().toString()
    };
  };

  components.forEach(component => {
    component.log = getLog(component);
  });
  autoTagComps.forEach(autoTagComp => {
    autoTagComp.log = getLog(autoTagComp);
    autoTagComp.log.message = `${autoTagComp.log.message} (bump dependencies versions)`;
  });
}

function setCurrentSchema(components, consumer) {
  if (consumer.isLegacy) return;
  components.forEach(component => {
    component.schema = _componentSchema().CURRENT_SCHEMA;
  });
}
/**
 * @todo: currently, there is only one function registered to the OnTag, which is the builder.
 * we set the extensions data and artifacts we got from the builder to the consumer-components.
 * however, if there is more than one function registered to the OnTag, the data will be overridden
 * by the last called function. when/if this happen, some kind of merge need to be done between the
 * results.
 */


function updateComponentsByTagResult(components, tagResult) {
  tagResult.forEach(result => {
    const matchingComponent = components.find(c => c.id.isEqual(result.id));

    if (matchingComponent) {
      const existingBuilder = matchingComponent.extensions.findCoreExtension(_constants().Extensions.builder);
      if (existingBuilder) existingBuilder.data = result.builderData.data;else matchingComponent.extensions.push(result.builderData);
    }
  });
}

function getPublishedPackages(components) {
  const publishedPackages = components.map(comp => {
    var _builderExt$data, _pkgData$data;

    const builderExt = comp.extensions.findCoreExtension(_constants().Extensions.builder);
    const pkgData = builderExt === null || builderExt === void 0 ? void 0 : (_builderExt$data = builderExt.data) === null || _builderExt$data === void 0 ? void 0 : _builderExt$data.aspectsData.find(a => a.aspectId === _constants().Extensions.pkg);
    return pkgData === null || pkgData === void 0 ? void 0 : (_pkgData$data = pkgData.data) === null || _pkgData$data === void 0 ? void 0 : _pkgData$data.publishedPackage;
  });
  return (0, RA().compact)(publishedPackages);
}

function addBuildStatus(consumer, components, buildStatus) {
  if (consumer.isLegacy) return;
  components.forEach(component => {
    component.buildStatus = buildStatus;
  });
}