"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

function _graphlib() {
  const data = _interopRequireWildcard(require("graphlib"));

  _graphlib = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _componentsList() {
  const data = _interopRequireDefault(require("../../consumer/component/components-list"));

  _componentsList = function () {
    return data;
  };

  return data;
}

function _index() {
  const data = require("../index");

  _index = function () {
    return data;
  };

  return data;
}

class Graph extends _graphlib().Graph {
  getSuccessorsByEdgeTypeRecursively(bitId, successorsList = [], visited = {}) {
    const successors = this.successors(bitId) || [];

    if (successors.length > 0 && !visited[bitId]) {
      successors.forEach(successor => {
        visited[bitId] = true;
        successorsList.push(successor);
        return this.getSuccessorsByEdgeTypeRecursively(successor, successorsList, visited);
      });
    }

    return successorsList;
  }
  /**
   * e.g. a graph has edges of "dependencies" and "devDependencies" and you want a new graph of the
   * dependencies only.
   */


  getSubGraphByEdgeType(edgeType) {
    const newGraph = new Graph();
    newGraph.setNodes(this.nodes());
    this.edges().forEach(edge => {
      const edgeLabel = this.edge(edge);

      if (edgeLabel === edgeType) {
        newGraph.setEdge(edge.v, edge.w, edgeType);
      }
    });
    return newGraph;
  }

  findSuccessorsInGraph(ids) {
    const dependenciesFromAllIds = _ramda().default.flatten(ids.map(id => this.getSuccessorsByEdgeTypeRecursively(id)));

    const components = _ramda().default.uniq([...dependenciesFromAllIds, ...ids]).map(id => this.node(id)).filter(val => val);

    return components;
  }
  /**
   * helps finding the versions of bit-ids using the components stored in the graph
   */


  getBitIdsIncludeVersionsFromGraph(ids, graph) {
    const components = graph.nodes().map(n => graph.node(n));
    return ids.map(id => {
      const component = components.find(c => c.id.isEqual(id)) || components.find(c => c.id.isEqualWithoutVersion(id));
      if (!component) throw new Error(`unable to find ${id.toString()} in the graph`);
      return component.id;
    });
  }

  toString() {
    return _graphlib().default.json.write(this);
  } // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


  static async buildGraphFromScope(scope) {
    const graph = new Graph();
    const allModelComponents = await scope.list();
    await this.addScopeComponentsAsNodes(allModelComponents, graph);
  }

  static _addDependenciesToGraph(id, graph, component) {
    const idStr = id.toString(); // save the full BitId of a string id to be able to retrieve it later with no confusion

    if (!graph.hasNode(idStr)) graph.setNode(idStr, id);
    Object.entries(component.depsIdsGroupedByType).forEach(([depType, depIds]) => {
      depIds.forEach(dependencyId => {
        const depIdStr = dependencyId.toString();
        if (!graph.hasNode(depIdStr)) graph.setNode(depIdStr, dependencyId);
        graph.setEdge(idStr, depIdStr, depType);
      });
    });
  }

  static async addScopeComponentsAsNodes(allModelComponents, graph, workspaceComponents, onlyLatest = false) {
    const scope = await (0, _index().loadScope)(process.cwd());
    await Promise.all(allModelComponents.map(async modelComponent => {
      const latestVersion = modelComponent.latest();
      const buildVersionP = modelComponent.listVersionsIncludeOrphaned().map(async versionNum => {
        if (onlyLatest && latestVersion !== versionNum) return;
        const id = modelComponent.toBitId().changeVersion(versionNum);
        const componentFromWorkspace = workspaceComponents ? workspaceComponents.find(comp => comp.id.isEqual(id)) : undefined;

        if (!componentFromWorkspace) {
          const componentVersion = await scope.getConsumerComponentIfExist(id);

          if (componentVersion) {
            // a component might be in the scope with only the latest version (happens when it's a nested dep)
            graph.setNode(componentVersion.id.toString(), componentVersion);
          }
        }
      });
      await Promise.all(buildVersionP);
    }));
  }

  static async buildGraphFromWorkspace(consumer, onlyLatest = false) {
    const componentsList = new (_componentsList().default)(consumer);
    const allModelComponents = await consumer.scope.list();
    const workspaceComponents = await componentsList.getFromFileSystem();
    const graph = new Graph();
    workspaceComponents.forEach(component => {
      const id = component.id.toString();
      graph.setNode(id, component);
    });
    await this.addScopeComponentsAsNodes(allModelComponents, graph, workspaceComponents, onlyLatest);

    _ramda().default.forEach(componentId => {
      const component = graph.node(componentId);
      Object.entries(component.depsIdsGroupedByType).forEach(([depType, depIds]) => {
        depIds.forEach(dependencyId => {
          const depIdStr = dependencyId.toString();

          if (graph.hasNode(depIdStr)) {
            graph.setEdge(componentId, depIdStr, depType);
          }
        });
      });
    }, graph.nodes());

    return graph;
  }

}

exports.default = Graph;