import { GraphQLClient } from 'graphql-request';
import fetch, { Response } from 'node-fetch';
import { HttpsProxyAgent } from 'https-proxy-agent';
import { Network } from '../network';
import { BitId, BitIds } from '../../../bit-id';
import Component from '../../../consumer/component';
import { ListScopeResult } from '../../../consumer/component/components-list';
import DependencyGraph from '../../graph/scope-graph';
import { LaneData } from '../../lanes/lanes';
import { ComponentLog } from '../../models/model-component';
import { ScopeDescriptor } from '../../scope';
import { ObjectList } from '../../objects/object-list';
import { FETCH_OPTIONS } from '../../../api/scope/lib/fetch';
import { PushOptions } from '../../../api/scope/lib/put';
import RemovedObjects from '../../removed-components';
export declare enum Verb {
    WRITE = "write",
    READ = "read"
}
export declare type AuthData = {
    type: string;
    credentials: string;
};
export declare const DEFAULT_AUTH_TYPE = "Bearer";
export declare class Http implements Network {
    private graphClient;
    private _token;
    private url;
    private scopeName;
    private proxyAgent?;
    constructor(graphClient: GraphQLClient, _token: string | undefined | null, url: string, scopeName: string, proxyAgent?: import("https-proxy-agent/dist/agent").default | undefined);
    static getToken(): string | null;
    static getProxyUrl(): string | undefined;
    get token(): string | null | undefined;
    close(): void;
    describeScope(): Promise<ScopeDescriptor>;
    deleteMany(ids: string[], force: boolean, context: Record<string, any>, idsAreLanes: boolean): Promise<RemovedObjects>;
    pushMany(objectList: ObjectList, pushOptions: PushOptions): Promise<string[]>;
    action<Options, Result>(name: string, options: Options): Promise<Result>;
    fetch(ids: string[], fetchOptions: FETCH_OPTIONS): Promise<ObjectList>;
    private getJsonResponse;
    private throwForNonOkStatus;
    private graphClientRequest;
    list(namespacesUsingWildcards?: string | undefined): Promise<ListScopeResult[]>;
    show(bitId: BitId): Promise<Component | null | undefined>;
    deprecateMany(ids: string[]): Promise<Record<string, any>[]>;
    undeprecateMany(ids: string[]): Promise<Record<string, any>[]>;
    log(id: BitId): Promise<ComponentLog[]>;
    latestVersions(bitIds: BitIds): Promise<string[]>;
    graph(): Promise<DependencyGraph>;
    listLanes(name?: string | undefined, mergeData?: boolean | undefined): Promise<LaneData[]>;
    private getHeaders;
    private addProxyAgentIfExist;
    static connect(host: string, scopeName: string): Promise<Http>;
}
export declare function getAuthHeader(token: string): {
    Authorization: string;
};
export declare function getFetcherWithProxy(): typeof fetch | ((url: any, opts: any) => Promise<Response>);
export declare function wrapFetcherWithProxy(proxyAgent: HttpsProxyAgent): (url: any, opts: any) => Promise<Response>;
export declare function getProxyAgent(proxy: string): HttpsProxyAgent;
export declare function getAuthDataFromHeader(authorizationHeader: string | undefined): AuthData | undefined;
