"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAuthHeader = getAuthHeader;
exports.getFetcherWithProxy = getFetcherWithProxy;
exports.wrapFetcherWithProxy = wrapFetcherWithProxy;
exports.getProxyAgent = getProxyAgent;
exports.getAuthDataFromHeader = getAuthDataFromHeader;
exports.Http = exports.DEFAULT_AUTH_TYPE = exports.Verb = void 0;

function _graphqlRequest() {
  const data = require("graphql-request");

  _graphqlRequest = function () {
    return data;
  };

  return data;
}

function _nodeFetch() {
  const data = _interopRequireDefault(require("node-fetch"));

  _nodeFetch = function () {
    return data;
  };

  return data;
}

function _httpsProxyAgent() {
  const data = require("https-proxy-agent");

  _httpsProxyAgent = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../../../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _component() {
  const data = _interopRequireDefault(require("../../../consumer/component"));

  _component = function () {
    return data;
  };

  return data;
}

function _globalFlags() {
  const data = _interopRequireDefault(require("../../../cli/global-flags"));

  _globalFlags = function () {
    return data;
  };

  return data;
}

function _globalConfig() {
  const data = require("../../../api/consumer/lib/global-config");

  _globalConfig = function () {
    return data;
  };

  return data;
}

function _constants() {
  const data = require("../../../constants");

  _constants = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../../../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

function _objectList() {
  const data = require("../../objects/object-list");

  _objectList = function () {
    return data;
  };

  return data;
}

function _remoteErrorHandler() {
  const data = require("../remote-error-handler");

  _remoteErrorHandler = function () {
    return data;
  };

  return data;
}

function _httpInvalidJsonResponse() {
  const data = require("../exceptions/http-invalid-json-response");

  _httpInvalidJsonResponse = function () {
    return data;
  };

  return data;
}

function _removedComponents() {
  const data = _interopRequireDefault(require("../../removed-components"));

  _removedComponents = function () {
    return data;
  };

  return data;
}

function _graphqlClientError() {
  const data = require("../exceptions/graphql-client-error");

  _graphqlClientError = function () {
    return data;
  };

  return data;
}

let Verb;
/**
 * fetched from HTTP Authorization header.
 * (see https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Authorization)
 */

exports.Verb = Verb;

(function (Verb) {
  Verb["WRITE"] = "write";
  Verb["READ"] = "read";
})(Verb || (exports.Verb = Verb = {}));

const DEFAULT_AUTH_TYPE = 'Bearer';
exports.DEFAULT_AUTH_TYPE = DEFAULT_AUTH_TYPE;

class Http {
  constructor(graphClient, _token, url, scopeName, proxyAgent) {
    this.graphClient = graphClient;
    this._token = _token;
    this.url = url;
    this.scopeName = scopeName;
    this.proxyAgent = proxyAgent;
  }

  static getToken() {
    const processToken = _globalFlags().default.token;

    const token = processToken || (0, _globalConfig().getSync)(_constants().CFG_USER_TOKEN_KEY);
    if (!token) return null;
    return token;
  }

  static getProxyUrl() {
    const proxyUrl = (0, _globalConfig().getSync)(_constants().CFG_HTTPS_PROXY) || (0, _globalConfig().getSync)(_constants().CFG_PROXY);
    return proxyUrl;
  }

  get token() {
    if (this._token === undefined) return this._token;
    return Http.getToken();
  }

  close() {}

  async describeScope() {
    const SCOPE_QUERY = (0, _graphqlRequest().gql)`
      {
        scope {
          name
        }
      }
    `;
    const data = await this.graphClientRequest(SCOPE_QUERY, Verb.READ);
    return {
      name: data.scope.name
    };
  }

  async deleteMany(ids, force, context, idsAreLanes) {
    const route = 'api/scope/delete';

    _logger().default.debug(`Http.delete, url: ${this.url}/${route}`);

    const body = JSON.stringify({
      ids,
      force,
      lanes: idsAreLanes
    });
    const headers = this.getHeaders({
      'Content-Type': 'application/json',
      'x-verb': 'write'
    });
    const opts = this.addProxyAgentIfExist({
      method: 'post',
      body,
      headers
    });
    const res = await (0, _nodeFetch().default)(`${this.url}/${route}`, opts);
    await this.throwForNonOkStatus(res);
    const results = await this.getJsonResponse(res);
    return _removedComponents().default.fromObjects(results);
  }

  async pushMany(objectList, pushOptions) {
    const route = 'api/scope/put';

    _logger().default.debug(`Http.pushMany, url: ${this.url}/${route}  total objects ${objectList.count()}`);

    const body = objectList.toTar();
    const headers = this.getHeaders({
      'push-options': JSON.stringify(pushOptions),
      'x-verb': Verb.WRITE
    });
    const opts = this.addProxyAgentIfExist({
      method: 'post',
      body,
      headers
    });
    const res = await (0, _nodeFetch().default)(`${this.url}/${route}`, opts);
    await this.throwForNonOkStatus(res);
    const ids = await this.getJsonResponse(res);
    return ids;
  }

  async action(name, options) {
    const route = 'api/scope/action';

    _logger().default.debug(`Http.action, url: ${this.url}/${route}`);

    const body = JSON.stringify({
      name,
      options
    });
    const headers = this.getHeaders({
      'Content-Type': 'application/json',
      'x-verb': Verb.WRITE
    });
    const opts = this.addProxyAgentIfExist({
      method: 'post',
      body,
      headers
    });
    const res = await (0, _nodeFetch().default)(`${this.url}/${route}`, opts);
    await this.throwForNonOkStatus(res);
    const results = await this.getJsonResponse(res);
    return results;
  }

  async fetch(ids, fetchOptions) {
    const route = 'api/scope/fetch';
    const scopeData = `scopeName: ${this.scopeName}, url: ${this.url}/${route}`;

    _logger().default.debug(`Http.fetch, ${scopeData}`);

    const body = JSON.stringify({
      ids,
      fetchOptions
    });
    const headers = this.getHeaders({
      'Content-Type': 'application/json',
      'x-verb': Verb.READ
    });
    const opts = this.addProxyAgentIfExist({
      method: 'post',
      body,
      headers
    });
    const res = await (0, _nodeFetch().default)(`${this.url}/${route}`, opts);

    _logger().default.debug(`Http.fetch got a response, ${scopeData}, status ${res.status}, statusText ${res.statusText}`);

    await this.throwForNonOkStatus(res);
    const objectList = await _objectList().ObjectList.fromTar(res.body);
    return objectList;
  }

  async getJsonResponse(res) {
    try {
      return await res.json();
    } catch (err) {
      _logger().default.error('failed response', res);

      throw new (_httpInvalidJsonResponse().HttpInvalidJsonResponse)(res.url);
    }
  }

  async throwForNonOkStatus(res) {
    var _jsonResponse, _jsonResponse$error, _jsonResponse2;

    if (res.ok) return;
    let jsonResponse;

    try {
      jsonResponse = await res.json();
    } catch (e) {// the response is not json, ignore the body.
    }

    _logger().default.error(`parsed error from HTTP, url: ${res.url}`, jsonResponse);

    const error = (_jsonResponse = jsonResponse) !== null && _jsonResponse !== void 0 && (_jsonResponse$error = _jsonResponse.error) !== null && _jsonResponse$error !== void 0 && _jsonResponse$error.code ? (_jsonResponse2 = jsonResponse) === null || _jsonResponse2 === void 0 ? void 0 : _jsonResponse2.error : jsonResponse;
    if (!error.message && jsonResponse.message) error.message = jsonResponse.message;
    const err = (0, _remoteErrorHandler().remoteErrorHandler)(error === null || error === void 0 ? void 0 : error.code, error, res.url, `url: ${res.url}. status: ${res.status}. text: ${res.statusText}`);
    throw err;
  }

  async graphClientRequest(query, verb = Verb.READ, variables) {
    _logger().default.debug(`http.graphClientRequest, scope "${this.scopeName}", url "${this.url}", query ${query}`);

    try {
      this.graphClient.setHeader('x-verb', verb);
      return await this.graphClient.request(query, variables);
    } catch (err) {
      if (err instanceof _graphqlRequest().ClientError) {
        throw new (_graphqlClientError().GraphQLClientError)(err, this.url, this.scopeName);
      } // should not be here. it's just in case


      throw err;
    }
  }

  async list(namespacesUsingWildcards) {
    const LIST_LEGACY = (0, _graphqlRequest().gql)`
      query listLegacy($namespaces: String) {
        scope {
          _legacyList(namespaces: $namespaces) {
            id
            deprecated
          }
        }
      }
    `;
    const data = await this.graphClientRequest(LIST_LEGACY, Verb.READ, {
      namespaces: namespacesUsingWildcards
    });

    data.scope._legacyList.forEach(comp => {
      comp.id = _bitId().BitId.parse(comp.id);
    });

    return data.scope._legacyList;
  }

  async show(bitId) {
    const SHOW_COMPONENT = (0, _graphqlRequest().gql)`
      query showLegacy($id: String!) {
        scope {
          _getLegacy(id: $id)
        }
      }
    `;
    const data = await this.graphClientRequest(SHOW_COMPONENT, Verb.READ, {
      id: bitId.toString()
    });
    return _component().default.fromString(data.scope._getLegacy);
  }

  async deprecateMany(ids) {
    const DEPRECATE_COMPONENTS = (0, _graphqlRequest().gql)`
      mutation deprecate($bitIds: [String!]!) {
        deprecate(bitIds: $bitIds) {
          bitIds
          missingComponents
        }
      }
    `;
    const res = await this.graphClientRequest(DEPRECATE_COMPONENTS, Verb.WRITE, {
      bitIds: ids
    });
    return res.deprecate;
  }

  async undeprecateMany(ids) {
    const UNDEPRECATE_COMPONENTS = (0, _graphqlRequest().gql)`
      mutation deprecate($bitIds: [String!]!) {
        undeprecate(bitIds: $bitIds) {
          bitIds
          missingComponents
        }
      }
    `;
    const res = await this.graphClientRequest(UNDEPRECATE_COMPONENTS, Verb.WRITE, {
      bitIds: ids
    });
    return res.undeprecate;
  }

  async log(id) {
    const GET_LOG_QUERY = (0, _graphqlRequest().gql)`
      query getLogs($id: String!) {
        scope {
          getLogs(id: $id) {
            message
            username
            email
            date
            hash
            tag
          }
        }
      }
    `;
    const data = await this.graphClientRequest(GET_LOG_QUERY, Verb.READ, {
      id: id.toString()
    });
    return data.scope.getLogs;
  }

  async latestVersions(bitIds) {
    const GET_LATEST_VERSIONS = (0, _graphqlRequest().gql)`
      query getLatestVersions($ids: [String]!) {
        scope {
          _legacyLatestVersions(ids: $ids)
        }
      }
    `;
    const data = await this.graphClientRequest(GET_LATEST_VERSIONS, Verb.READ, {
      ids: bitIds.map(id => id.toString())
    });
    return data.scope._legacyLatestVersions;
  }

  graph() {
    throw new Error('Method not implemented.');
  } // TODO: ran (TBD)


  async listLanes(name, mergeData) {
    const LIST_LANES = (0, _graphqlRequest().gql)`
    query listLanes() {
      lanes {
        list()
      }
    }
    `;
    const res = await this.graphClientRequest(LIST_LANES, Verb.READ, {
      mergeData
    });
    return res.lanes.list;
  }

  getHeaders(headers = {}) {
    const authHeader = this.token ? getAuthHeader(this.token) : {};
    return Object.assign(headers, authHeader);
  }

  addProxyAgentIfExist(opts = {}) {
    const optsWithProxy = this.proxyAgent ? Object.assign({}, opts, {
      agent: this.proxyAgent
    }) : opts;
    return optsWithProxy;
  }

  static async connect(host, scopeName) {
    const token = Http.getToken();
    const headers = token ? getAuthHeader(token) : {};
    const proxyUrl = Http.getProxyUrl();
    const proxyAgent = proxyUrl ? getProxyAgent(proxyUrl) : undefined;
    const graphClient = new (_graphqlRequest().GraphQLClient)(`${host}/graphql`, {
      headers,
      fetch: getFetcherWithProxy()
    });
    return new Http(graphClient, token, host, scopeName, proxyAgent);
  }

}

exports.Http = Http;

function getAuthHeader(token) {
  return {
    Authorization: `${DEFAULT_AUTH_TYPE} ${token}`
  };
}
/**
 * Read the proxy config from the global config, and wrap fetch with fetch with proxy
 */


function getFetcherWithProxy() {
  const proxyUrl = Http.getProxyUrl();
  const proxyAgent = proxyUrl ? getProxyAgent(proxyUrl) : undefined;
  const fetcher = proxyAgent ? wrapFetcherWithProxy(proxyAgent) : _nodeFetch().default;
  return fetcher;
}
/**
 * return a fetch wrapper with the proxy agent inside
 * @param proxyAgent
 */


function wrapFetcherWithProxy(proxyAgent) {
  return (url, opts) => {
    const actualOpts = Object.assign({}, opts, {
      agent: proxyAgent
    });
    return (0, _nodeFetch().default)(url, actualOpts);
  };
}

function getProxyAgent(proxy) {
  const proxyAgent = new (_httpsProxyAgent().HttpsProxyAgent)(proxy);
  return proxyAgent;
}

function getAuthDataFromHeader(authorizationHeader) {
  if (!authorizationHeader) return undefined;
  const authorizationSplit = authorizationHeader.split(' ');

  if (authorizationSplit.length !== 2) {
    throw new Error(`fatal: HTTP Authorization header "${authorizationHeader}" is invalid. it should have exactly one space`);
  }

  return {
    type: authorizationSplit[0],
    credentials: authorizationSplit[1]
  };
}