"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.multipleVersionDependenciesToConsumer = multipleVersionDependenciesToConsumer;
exports.default = void 0;

function _defineProperty2() {
  const data = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

  _defineProperty2 = function () {
    return data;
  };

  return data;
}

function _pMapSeries() {
  const data = _interopRequireDefault(require("p-map-series"));

  _pMapSeries = function () {
    return data;
  };

  return data;
}

function _ramda() {
  const data = _interopRequireDefault(require("ramda"));

  _ramda = function () {
    return data;
  };

  return data;
}

function _bitId() {
  const data = require("../bit-id");

  _bitId = function () {
    return data;
  };

  return data;
}

function _componentDependencies() {
  const data = _interopRequireDefault(require("./component-dependencies"));

  _componentDependencies = function () {
    return data;
  };

  return data;
}

function _dependenciesNotFound() {
  const data = require("./exceptions/dependencies-not-found");

  _dependenciesNotFound = function () {
    return data;
  };

  return data;
}

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2().default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

class VersionDependencies {
  constructor(component, dependencies, sourceScope, version) {
    this.component = component;
    this.dependencies = dependencies;
    this.sourceScope = sourceScope;
    this.version = version;
  }

  get allDependencies() {
    return this.dependencies;
  }

  get allDependenciesIds() {
    return _bitId().BitIds.fromArray(this.dependencies.map(dep => dep.id));
  }

  getMissingDependencies() {
    const allDepsIds = this.allDependenciesIds;
    return this.version.flattenedDependencies.filter(id => !allDepsIds.has(id));
  }

  throwForMissingDependencies() {
    const missing = this.getMissingDependencies();

    if (missing.length) {
      throw new (_dependenciesNotFound().DependenciesNotFound)(this.component.id.toString(), missing.map(m => m.toString()));
    }
  }

  async toConsumer(repo, manipulateDirData) {
    const depToConsumer = dep => dep.toConsumer(repo, manipulateDirData);

    const dependenciesP = Promise.all(this.dependencies.map(depToConsumer));
    const componentP = this.component.toConsumer(repo, manipulateDirData);
    const [component, dependencies] = await Promise.all([componentP, dependenciesP]);
    return new (_componentDependencies().default)({
      component,
      dependencies,
      devDependencies: [],
      extensionDependencies: [],
      missingDependencies: this.getMissingDependencies()
    });
  }

  async collectObjects(repo, clientVersion, options) {
    // for the dependencies, don't collect parents they might not exist
    const depOptions = _objectSpread(_objectSpread({}, options), {}, {
      collectParents: false
    });

    const depsP = (0, _pMapSeries().default)(this.allDependencies, dep => dep.collectObjects(repo, clientVersion, depOptions));
    const compP = this.component.collectObjects(repo, clientVersion, options);
    const [component, dependencies] = await Promise.all([compP, depsP]);
    return [...component, ..._ramda().default.flatten(dependencies)];
  }

}

exports.default = VersionDependencies;

async function multipleVersionDependenciesToConsumer(versionDependencies, repo) {
  const flattenedCompVer = {};
  const flattenedConsumerComp = {};
  versionDependencies.forEach(verDep => {
    const allComps = [verDep.component, ...verDep.dependencies];
    allComps.forEach(compVer => {
      flattenedCompVer[compVer.id.toString()] = compVer;
    });
  });
  await Promise.all(Object.keys(flattenedCompVer).map(async idStr => {
    flattenedConsumerComp[idStr] = await flattenedCompVer[idStr].toConsumer(repo, null);
  }));
  return versionDependencies.map(verDep => {
    return new (_componentDependencies().default)({
      component: flattenedConsumerComp[verDep.component.id.toString()],
      dependencies: verDep.dependencies.map(d => flattenedConsumerComp[d.id.toString()]),
      devDependencies: [],
      extensionDependencies: [],
      missingDependencies: verDep.getMissingDependencies()
    });
  });
}