"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.promise.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getLastModifiedDirTimestampMs = getLastModifiedDirTimestampMs;
exports.getLastModifiedPathsTimestampMs = getLastModifiedPathsTimestampMs;
exports.getLastModifiedComponentTimestampMs = getLastModifiedComponentTimestampMs;

function _glob() {
  const data = _interopRequireDefault(require("glob"));

  _glob = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _ramdaAdjunct() {
  const data = require("ramda-adjunct");

  _ramdaAdjunct = function () {
    return data;
  };

  return data;
}

/**
 * check recursively all the sub-directories as well
 */
async function getLastModifiedDirTimestampMs(rootDir) {
  const allDirs = _glob().default.sync(`${rootDir}/**/`); // the trailing slash instructs glob to show only dirs


  return getLastModifiedPathsTimestampMs(allDirs);
}

async function getLastModifiedPathsTimestampMs(paths) {
  const pathsStats = await Promise.all(paths.map(dir => getPathStatIfExist(dir)));
  const statsWithoutNull = (0, _ramdaAdjunct().compact)(pathsStats);
  const timestamps = statsWithoutNull.map(stat => stat.mtimeMs);
  return Math.max(...timestamps);
}

async function getLastModifiedComponentTimestampMs(rootDir, files) {
  const lastModifiedDirs = await getLastModifiedDirTimestampMs(rootDir);
  const lastModifiedFiles = await getLastModifiedPathsTimestampMs(files);
  return Math.max(lastModifiedDirs, lastModifiedFiles);
}

async function getPathStatIfExist(path) {
  try {
    return await _fsExtra().default.stat(path);
  } catch (err) {
    if (err.code === 'ENOENT') return null;
    throw err;
  }
}